package Logoved::DB::Rule::Matcher::Sequence;
use Moo;
extends 'Logoved::DB::Rule::Matcher';

has 'seq_pattern' => (is => 'ro');
has 'groups' => (is => 'rw', default => sub { [] } );

has 'lineno_begin' => (is => 'rw');
has 'lineno_end' => (is => 'rw');
has 'line' => (is => 'rw');
has '__next_pattern_to_match' => (is => 'rw', default => 0);

our $verbose=0;

sub match {
    my ($self, $line, $chomped_line)=@_;
    my $source=$self->source;
    my $seq_pattern=$self->seq_pattern;
    my $next_pattern_to_match=$self->__next_pattern_to_match;
    #warn 'Logoved: SEQ: ',$next_pattern_to_match?$next_pattern_to_match:'0',' line=',$line;
    if (!$next_pattern_to_match) {
	my @begin_groups= $chomped_line =~ $seq_pattern->[0];
	if (@begin_groups>0) {
	    $self->lineno_begin($source->lineno);
	    $self->line($line);
	    $self->groups(\@begin_groups) if $#+ >0;
	    warn 'Logoved: SEQBEGIN ',$self->__next_pattern_to_match,': ',$source->name, ' ',$source->file, ': ', $line if $verbose;
	    $self->__next_pattern_to_match(1);
	}
    } else {
	my $pattern=$seq_pattern->[$next_pattern_to_match];
	die "Oops! undefined pattern at ",$next_pattern_to_match,' rule=',$self->rule->subpath unless defined $pattern;
	my @_groups=$chomped_line =~ $seq_pattern->[$next_pattern_to_match];
	if (@_groups==0) {
	    # not a match:
	    warn 'Logoved: SEQ BODY: not a match: ', $source->name, ' ',$source->file, ': ', $line if $verbose;
	    $self->__next_pattern_to_match(0);
	} else {
	    $self->line($self->line() . $line);
	    # regex has $#+ groups, see @+, @LAST_MATCH_END
	    # http://perldoc.perl.org/perlvar.html#@LAST_MATCH_END
	    push @{$self->groups}, @_groups if $#+ >0;
	    warn 'Logoved: SEQ BODY ',$next_pattern_to_match,': ', $source->name, ' ',$source->file, ': ', $line if $verbose;
	    $self->__next_pattern_to_match($next_pattern_to_match+1);
	    if ($next_pattern_to_match>=$#{$seq_pattern}) {
		$self->__next_pattern_to_match(0);
		$self->lineno_end($source->lineno);
		warn 'Logoved: SEQEND: ', $source->name, ' ',$source->file, ': ', $line if $verbose;
		my @args=(
		    rule => $self->rule,
		    logfile => $source->file,
		    line => $self->line,
		    lineno => $self->lineno_begin.'-'.$self->lineno_end,
		    );
		my $groups=$self->groups;
		push @args, groups => $groups if $groups and @$groups;
		return Logoved::DB::Rule::Match->new(@args);
	    }
	}
    }
    return;
}

1;
