package Logoved::DB::Rule::MatcherFactory;
use Logoved::DB::Rule::Matcher::Pattern;
use Logoved::DB::Rule::Matcher::Sequence;
use Logoved::DB::Rule::Matcher::Multiline;
use Moo;
has 'rule' => (is => 'ro');
has 'source' => (is => 'lazy');
has 'matcher' => (is => 'lazy');
has 'rule_subpath2matchers_cache' => (is => 'ro', default => sub { {} } );

sub get_matcher {
    my ($self,$rule,$source)=@_;
    my $subpath=$rule->subpath;
    my $cache=$self->rule_subpath2matchers_cache;
    my $cached_matcher=$cache->{$subpath};
    return $cached_matcher if $cached_matcher;
    my $matcher=$self->__get_matcher_raw($rule,$source);
    $cache->{$subpath}=$matcher;
    return $matcher;
}

sub __get_matcher_raw {
    my ($self,$rule,$source)=@_;
    my @args=(rule => $rule, source => $source);

    my $patterns_ref=$rule->pattern;
    if ($patterns_ref and @$patterns_ref) {
	my $matcher=Logoved::DB::Rule::Matcher::Pattern->new(
	    @args, patterns=>$patterns_ref
	    );
	return $matcher;
    }

    my $seq_pattern_ref=$rule->pattern_seq;
    if ($seq_pattern_ref and @$seq_pattern_ref) {
	my $matcher=Logoved::DB::Rule::Matcher::Sequence->new(
	    @args, seq_pattern=>$seq_pattern_ref
	    );
	return $matcher;
    }

    my $multiline_pattern_begin=$rule->multiline_pattern_begin;
    if ($multiline_pattern_begin) {
	return Logoved::DB::Rule::Matcher::Multiline->new(
	    @args,
	    multiline_pattern_begin => $multiline_pattern_begin,
	    multiline_pattern_body => $rule->multiline_pattern_body,
	    multiline_pattern_end => $rule->multiline_pattern_end,
	    );
    }
    #use Data::Dumper; print Dumper($rule);
    die "Bad rule: ", $rule->name;
}

1;
