package Logoved::DB::Stream::Listener::Factory;
use Moo;
use Logoved::DB::Rule::MatcherFactory;
use Logoved::DB::Stream::Out::Listener;
use Logoved::DB::Stream::Out::MainListener;
use Logoved::DB::Stream::Out::FindReqListener;

use Logoved::Stream 0.025;
with 'Logoved::Role::Stream::Listener::Factory';

has 'matcher_factory' => (is => 'rw', );
has 'match_handler' => (is => 'rw', );
has 'current_file' => (is => 'rw', );
has 'current_status' => (is => 'rw', );
has 'matched' => (is => 'rw', default => sub { [] });
has 'not_matched' => (is => 'rw', default => sub { [] });
has 'successful' => (is => 'rw', default => sub { [] });
has 'matches' => (is => 'rw', default => sub { [] });
has 'ruleset' => (is => 'ro', );

my %scope_match_section=(
    'apt' => {
	'hasher::chroot::apt' => 1,
	'hasher::chroot::file_conflict' => 1,
    },
#    'rpmbuild-errors' => {'rpmbuild::error' => 1},
    'run' => {
		'rpmbuild::prep' => 1,
		'rpmbuild::build' => 1,
		'rpmbuild::install' => 1,
		'rpmbuild::check' => 1,
		'rpmbuild::error' => 1,
	    },
    'prep' => {
		'rpmbuild::prep' => 1,
		'rpmbuild::build' => 1,
		'rpmbuild::install' => 1,
		'rpmbuild::error' => 1,
	    },
    'conf' => {
		'rpmbuild::prep' => 1,
		'rpmbuild::build' => 1,
#		'rpmbuild::install' => 1,
		'rpmbuild::error' => 1,
	    },
    'build' => {
		'rpmbuild::prep' => 1,
		'rpmbuild::build' => 1,
		'rpmbuild::install' => 1,
		'rpmbuild::error' => 1,
	    },
    'check' => {
		'rpmbuild::build' => 1,
		'rpmbuild::install' => 1,
		'rpmbuild::check' => 1,
		'rpmbuild::error' => 1,
	    },
    );

sub is_rule_match_section {
    my ($self,$rule,$section_type)=@_;
    my @scope=('run');
    my $scope_ref=$rule->scope;
    @scope=@$scope_ref if $scope_ref and @$scope_ref;
    foreach my $scope (@scope) {
	return 1 if $scope eq $section_type;
	next if $scope=~/^(?:hasher|rpmbuild|beehive)::/;
	my $section_check=$scope_match_section{$scope};
	die "Oops: unsupported scope: $scope in ", $rule->subpath unless $section_check;
	return 1 if $section_check->{$section_type};
    }
    return 0;
}

sub get_listeners_for {
    my ($self, $stream)=@_;
    my $rules_ref=$self->ruleset->rules;
    my $section_type=$stream->section_type;
    return [Logoved::DB::Stream::Out::MainListener->new(
	listeners_factory => $self,
	stream => $stream,
	)] if $section_type eq '';
    my $source=$stream->source;
    my $class='Logoved::DB::Stream::Out::Listener';
    $class='Logoved::DB::Stream::Out::FindReqListener' if $section_type eq 'rpmbuild::files::find_requires';
    my @matchers;
    my $matcher_factory=$self->matcher_factory;
    foreach my $rule (@$rules_ref) {
	#warn 'rule ',$rule->subpath, ' match ',$section_type,'=',$self->is_rule_match_section($rule,$section_type),"\n";
	push @matchers, $matcher_factory->get_matcher($rule,$source)
	    if $self->is_rule_match_section($rule,$section_type);
    }
    return [$class->new(
	    listeners_factory => $self,
	    stream => $stream,
	    matchers => \@matchers,
	    )] if @matchers;
    return [];
}

sub init_new_stream {
    my ($self,$stream)=@_;
    $self->current_file($stream->file);
    $self->matcher_factory(Logoved::DB::Rule::MatcherFactory->new());
}

sub done_new_stream {
    my ($self)=@_;
    my $file=$self->current_file;
    my $current_status=$self->current_status;
    my ($successful)=($current_status and $current_status eq 'success');
    push @{$self->successful}, $file if $successful;
    my $matches=$self->matches;
    if (@$matches>0) {
	my $handler=$self->match_handler;
	&$handler($self,$file,$matches) if $handler;
	$self->matches([]);
	push @{$self->matched}, $file;
    } else {
	push @{$self->not_matched}, $file if not $successful;
    }
    $self->current_file(undef);
}

sub add_matches {
    my ($self,$matches)=@_;
    return unless $matches and @$matches;
    push @{$self->matches}, @$matches;
}

sub main_status {
    my ($self,$status)=@_;
    $self->current_status($status);
}

1;
